#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# ontoScorer: Script to compare two ontologies and calculate metrics 
#------------------------------------------------------------------------------
# This script loads two OWL ontologies from local files, extracts classes from 
# these ontologies and compares them. It then calculates precision, recall, and
# F1 score, which are commonly used metrics in information retrieval to assess 
# the quality of a retrieval system. The paths to the ontology files are 
# specified directly in the script.
#==============================================================================

from rdflib import Graph, OWL
from rdflib.namespace import split_uri
from sklearn.metrics import precision_score, recall_score, f1_score

# Define the path to the data folder
DATA_FOLDER_PATH = "data"

# Define paths to the ontology files
REFERENCE_ONTOLOGY_PATH = f"{DATA_FOLDER_PATH}/reference_ontology.ttl"
GENERATED_ONTOLOGY_PATH = f"{DATA_FOLDER_PATH}/generated_ontology.ttl"


#==============================================================================
# Methods to manipulate ontologies and calculate the metrics
#==============================================================================  

def load_ontology(path):
    g = Graph()
    g.parse(path, format="ttl")
    return g


def get_classes(ontology, ontology_path):
    classes = set()
    triplets_count = 0
    for s, p, o in ontology.triples((None, None, None)):
        triplets_count += 1
        if o == OWL.Class:
            _, class_name = split_uri(s)
            classes.add(class_name)
    print(f"Ontology analyzed: {ontology_path}")
    print(f"Number of classes found: {len(classes)}")
    print(f"Number of triplets analyzed: {triplets_count}")
    return classes


def calculate_metrics(reference_classes, generated_classes):
    all_classes = reference_classes.union(generated_classes)
    y_true = [1 if cls in reference_classes else 0 for cls in all_classes]
    y_pred = [1 if cls in generated_classes else 0 for cls in all_classes]

    precision = precision_score(y_true, y_pred)
    recall = recall_score(y_true, y_pred)
    f1 = f1_score(y_true, y_pred)

    return precision, recall, f1


#==============================================================================
# Main Method
#==============================================================================  

def main():
    # Load ontologies
    ref_ontology = load_ontology(REFERENCE_ONTOLOGY_PATH)
    gen_ontology = load_ontology(GENERATED_ONTOLOGY_PATH)

    # Extract classes
    ref_classes = get_classes(ref_ontology, REFERENCE_ONTOLOGY_PATH)
    gen_classes = get_classes(gen_ontology, GENERATED_ONTOLOGY_PATH)

    # Calculate metrics
    precision, recall, f1 = calculate_metrics(ref_classes, gen_classes)

    # Print results
    print(f"Precision: {precision}")
    print(f"Recall: {recall}")
    print(f"F1 Score: {f1}")

if __name__ == "__main__":
    main()

