#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

"""
ontoScorer: Batch evaluation of ontology pairs.
------------------------------------------------------------------------------
This script processes multiple pairs of reference and generated ontologies, computes comparison metrics, 
and produces reports for each pair as well as a summary report for all pairs.
"""

import os
from ontoScorer.scorer_batch import ScorerBatch

#--------------------------------------------------------------------------
# Parameters
#--------------------------------------------------------------------------

# Directory containing the ontologies
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
DATA_FOLDER_PATH = os.path.join(SCRIPT_DIR, "data")
OUTPUT_FOLDER_PATH = os.path.join(SCRIPT_DIR, "output_reports")

# List of ontology pairs (reference, generated) for evaluation.
# Ideally, these paths would be dynamically generated or loaded from some configuration.
ONTOLOGY_PAIRS = [
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_1.ttl", 
     f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_1.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_2.ttl", 
     f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_2.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_3.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_3.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_4.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_4.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_5.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_5.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_6.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_6.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_7.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_7.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_8.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_8.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_9.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_9.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_10.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_10.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_11.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_11.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_12.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_12.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_13.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_13.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_14.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_14.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_15.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_15.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_16.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_16.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_17.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_17.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_18.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_18.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_19.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_19.ttl"),
    (f"{DATA_FOLDER_PATH}/reference_ontology/SolarSystemProd_factoid_20.ttl", 
      f"{DATA_FOLDER_PATH}/generated_ontology/SolarSystemProd_factoid_20.ttl")    
]

# Prefixes used to harmonize or align concepts in the ontologies
EQUIVALENT_PREFIX = [
    ("base", "https://reference.tetras-libre.fr/base-ontology#", "https://tenet.tetras-libre.fr/base-ontology#"),
    ("result", "https://reference.tetras-libre.fr/expected-result#", "https://tenet.tetras-libre.fr/extract-result#")
] 


#--------------------------------------------------------------------------
# Useful Method(s)
#--------------------------------------------------------------------------

def common_prefix(strings):
    """ Return the longest common prefix of the list of strings. """
    if not strings:
        return ''
    
    prefix = strings[0]
    for string in strings:
        while string[:len(prefix)] != prefix:
            prefix = prefix[:-1]
    return prefix

def common_suffix(strings):
    """ Return the longest common suffix of the list of strings. """
    if not strings:
        return ''
    
    suffix = strings[0]
    for string in strings:
        while string[-len(suffix):] != suffix:
            suffix = suffix[1:]
    return suffix

def build_ontology_pairs(reference_subfolder_name, generated_subfolder_name):
    """
    Build pairs of ontology files from two subfolders based on their filename similarity.

    Args:
    - reference_subfolder_name (str): Name of the reference ontology subfolder.
    - generated_subfolder_name (str): Name of the generated ontology subfolder.

    Returns:
    - list of tuples: Each tuple contains a pair of file paths (reference, generated).
    """

    reference_subfolder = os.path.join(DATA_FOLDER_PATH, reference_subfolder_name)
    generated_subfolder = os.path.join(DATA_FOLDER_PATH, generated_subfolder_name)

    reference_files = sorted(os.listdir(reference_subfolder))
    generated_files = sorted(os.listdir(generated_subfolder))

    pairs = []

    # Identify the common prefix and suffix for reference files
    ref_common_prefix = common_prefix(reference_files)
    ref_common_suffix = common_suffix(reference_files)

    # Identify the common prefix and suffix for generated files
    gen_common_prefix = common_prefix(generated_files)
    gen_common_suffix = common_suffix(generated_files)

    # Extract the varying parts from filenames
    ref_variants = [f[len(ref_common_prefix):-len(ref_common_suffix)] for f in reference_files]
    gen_variants = [f[len(gen_common_prefix):-len(gen_common_suffix)] for f in generated_files]

    # Pair up matching variants
    for r_variant, r_file in zip(ref_variants, reference_files):
        if r_variant in gen_variants:
            # Use the actual file from the generated_files list
            g_file_index = gen_variants.index(r_variant)
            g_file = generated_files[g_file_index]
            pairs.append((os.path.join(reference_subfolder, r_file),
                          os.path.join(generated_subfolder, g_file)))

    return pairs


#--------------------------------------------------------------------------
# Main Method
#--------------------------------------------------------------------------

def main():
    print("'\n === OntoScorer Run === '")
    
    
    print("\n" + "-- Initialization")
    
    print("----- Building ontology pairs...")
    ontology_pairs = ONTOLOGY_PAIRS
    # ontology_pairs = build_ontology_pairs('reference_ontology', 'generated_ontology')
    
    print(f"----- Found {len(ontology_pairs)} ontology pair{'s' if len(ontology_pairs) > 1 else ''}.")

    print("----- Initializing ScorerBatch...")
    scorer_batch = ScorerBatch(ontology_pairs, EQUIVALENT_PREFIX, OUTPUT_FOLDER_PATH)
    
    
    print("\n" + "-- Analyze")
    
    print("----- Computing metrics for all ontology pairs...")
    scorer_batch.compute_all_metrics()
    
    print("\n" + "-- Report(s)")
    
    print("----- Generating individual reports for each ontology pair...")
    scorer_batch.generate_all_reports()
    
    print("----- Generating summary report...")
    scorer_batch.generate_summary_report()
    
    print("\n" + "-- Process completed!")

if __name__ == "__main__":
    main()
