# -*-coding:Utf-8 -*

"""
metric_score: Ontology Scoring Helper Module
------------------------------------------------------------------------------
This module defines the Score class, which helps to encapsulate and compute 
individual scoring metrics such as precision, recall, and F1 score. It also 
keeps track of the number of total elements and matched elements.
"""

from sklearn.metrics import precision_score, recall_score, f1_score


class Score:
    """Class to encapsulate individual scoring metrics."""
        
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------

    def __init__(self):
        """Initialize an empty score object with default metrics."""
        self.precision = None
        self.recall = None
        self.f1 = None
        self.total_elements = 0
        self.matched_elements = 0
        self.y_true = []
        self.y_pred = []


    #--------------------------------------------------------------------------
    # Computing Method(s)
    #--------------------------------------------------------------------------
    
    def compute(self, y_true, y_pred):
        """
        Compute and update the precision, recall, and F1 score based on true and predicted labels.

        Args:
            y_true (list[int]): List of ground truth (correct) labels.
            y_pred (list[int]): List of predicted labels.

        Returns:
            None
        """
        self.y_true = y_true
        self.y_pred = y_pred
        if len(y_true) > 0:
            self.precision = precision_score(y_true, y_pred, zero_division=1)
            self.recall = recall_score(y_true, y_pred)
            self.f1 = f1_score(y_true, y_pred)
            self.total_elements = len(y_true)
            self.matched_elements = sum([1 for true, pred in zip(y_true, y_pred) if true == pred])
        

    @staticmethod
    def aggregate_f1(score_objects):
        """Compute aggregated F1 score from multiple Score objects."""
        aggregated_score = Score()
        
        all_y_true = []
        all_y_pred = []
        
        for score in score_objects:
            all_y_true.extend(score.y_true)
            all_y_pred.extend(score.y_pred)
    
        aggregated_score.compute(all_y_true, all_y_pred)
    
        return aggregated_score
        
    
    #--------------------------------------------------------------------------
    # Printing Method(s)
    #--------------------------------------------------------------------------
    
    def _format_metric(self, metric_value):
        return f"{metric_value:.4f}" if metric_value is not None else "NA"

    def __str__(self):
        metrics = [
        f"\tPrecision: {self._format_metric(self.precision)}",
        f"\tRecall: {self._format_metric(self.recall)}",
        f"\tF1 Score: {self._format_metric(self.f1)}",
        f"\tTotal Elements: {self.total_elements}",
        f"\tMatched Elements: {self.matched_elements}"
        ]
        return "\n".join(metrics)
