#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# Ontology Analyzer
#------------------------------------------------------------------------------
# Extracts various entities (Classes, Object Properties, Data Properties, 
# Individuals, Annotations) from RDF/OWL ontologies. Enables foundational 
# comparisons between different ontologies.
#==============================================================================

from rdflib import Graph, RDF, RDFS, OWL
from rdflib import URIRef, BNode
from rdflib.namespace import split_uri

from ontology_utils import harmonize_prefixes, uri_to_short_form, extract_prefixes_from_graph

from ontology_metadata import Metadata
from ontoScorer.ontology_entity import (Entity, NamedEntity, BlankEntity)
from ontoScorer.ontology_taxonomic_relations import (SubclassOfRelation, 
                                                     SubpropertyOfRelation, 
                                                     InstanceOfRelation)
from ontoScorer.ontology_nontaxonomic_relations import (ObjectPropertyRelation, 
                                                        DataPropertyRelation)
from ontoScorer.ontology_axioms import (Axiom, 
                                        RestrictionAxiom,
                                        SomeValuesFromRestrictionAxiom)


#==============================================================================
# Class: Ontology
#==============================================================================

class Ontology:
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
    
    def __init__(self, ontology_path, equivalent_prefix=None):
        """Initialize the Ontology object."""
        self.path = ontology_path
        self.equivalent_prefix = equivalent_prefix if equivalent_prefix else []
        self.graph = self.load_ontology(ontology_path)
        self.classes = self.get_classes()


    #--------------------------------------------------------------------------
    # Base Method(s)
    #--------------------------------------------------------------------------
    
    def load_ontology(self, path):
        """Load the ontology from the given path into an RDF graph."""
        g = Graph()
        g.parse(path, format="ttl")
        g = self.harmonize_ontology_prefixes(g)
        return g

    def load_ontology(self, path):
        """Load the ontology from the given path into an RDF graph."""
        g = Graph()
        g.parse(path, format="ttl")
        g = self.harmonize_ontology_prefixes(g)
        return g

    def harmonize_ontology_prefixes(self, graph):
        if self.equivalent_prefix:
            for prefix, source_uri, target_uri in self.equivalent_prefix:
                graph = harmonize_prefixes(graph, source_uri, target_uri)
        return graph

    def get_uri_short_form(self, uri):
        graph_prefixes = extract_prefixes_from_graph(self.graph)
        return uri_to_short_form(uri, self.equivalent_prefix, graph_prefixes)
    
    
    def _get_elements_of_type(self, rdf_type):
        """Extract all entities of a specific RDF type from the ontology."""
        entities = []
        for s, _, o in self.graph.triples((None, RDF.type, rdf_type)):
            if isinstance(s, BNode):
                entities.append(BlankEntity(s, self.graph))
            elif isinstance(s, URIRef):
                entities.append(NamedEntity(s, self.graph))
        return entities
    

    #--------------------------------------------------------------------------
    # Metadata and Annotation Extraction
    #--------------------------------------------------------------------------

    def extract_metadata(self):
        """Extract metadata from the ontology and return a Metadata object."""
        version_info = self._get_version_info()  
        namespaces = list(self.graph.namespaces())
        return Metadata(version_info, namespaces)
    
    def _get_version_info(self):
        pass # TODO
        return -1

    def get_annotations(self):
        """Extract all annotation comments from the ontology."""
        annotations = set()
        for _, _, o in self.graph.triples((None, RDFS.label, None)):
            annotations.add(str(o))
        return annotations


    #--------------------------------------------------------------------------
    # Entity Extraction
    #--------------------------------------------------------------------------
    
    def get_classes(self):
        """Extract all classes from the ontology."""
        return self._get_elements_of_type(OWL.Class)

    def get_object_properties(self):
        """Extract all object properties from the ontology."""
        return self._get_elements_of_type(OWL.ObjectProperty)

    def get_data_properties(self):
        """Extract all data properties from the ontology."""
        return self._get_elements_of_type(OWL.DatatypeProperty)

    def get_individuals(self) -> list:
        """Extract all individuals from the ontology, including entities explicitly typed as owl:Individual, 
        owl:NamedIndividual or any class."""
        
        individuals = self._get_elements_of_type(OWL.NamedIndividual)
        individuals += self._get_elements_of_type(URIRef("http://www.w3.org/2002/07/owl#Individual"))
    
        all_classes = {cls.reference for cls in self.get_classes()}
        for cls in all_classes:
            individuals += [NamedEntity(s, self.graph) if isinstance(s, URIRef) else BlankEntity(s, self.graph)
                            for s, _, o in self.graph.triples((None, RDF.type, cls))]
    
        return list(set(individuals))  # Ensuring uniqueness
    
    
    #--------------------------------------------------------------------------
    # Taxonomic Relation Extraction (Hierarchical Structure)
    #--------------------------------------------------------------------------

    def get_subclass_relations(self):
        relations = []
        for s, _, o in self.graph.triples((None, RDFS.subClassOf, None)):
            if not (isinstance(s, BNode) or isinstance(o, BNode)):
                relations.append(SubclassOfRelation(s, o, self.graph))
        return relations
    
    def get_subproperty_relations(self):
        relations = []
        for s, _, o in self.graph.triples((None, RDFS.subPropertyOf, None)):
            if not (isinstance(s, BNode) or isinstance(o, BNode)):
                relations.append(SubpropertyOfRelation(s, o, self.graph))
        return relations
    
    def get_instance_relations(self):
        relations = []
        for s, _, o in self.graph.triples((None, RDF.type, None)):
            # Filter out the RDF schema and ontology types to get only the actual instances
            if o not in [RDF.Property, RDFS.Class, OWL.Class, OWL.ObjectProperty, OWL.DatatypeProperty] and not (isinstance(s, BNode) or isinstance(o, BNode)):
                relations.append(InstanceOfRelation(s, o, self.graph))
        return relations


    def get_taxonomic_relations(self):
        return self.get_subclass_relations() + self.get_subproperty_relations() + self.get_instance_relations()
    

    #--------------------------------------------------------------------------
    # Non-Taxonomic Relation Extraction
    #--------------------------------------------------------------------------
        
    def get_object_property_relations(self):
        """Extract all object property relations which aren't taxonomically classified."""
        relations = []
        object_properties = {prop.reference for prop in self.get_object_properties()}
        
        for prop in object_properties:
            for s, _, o in self.graph.triples((None, prop, None)):
                relation = ObjectPropertyRelation(s, prop, o, self.graph)
                relations.append(relation)
                
        return relations
    
    def get_data_property_relations(self):
        """Extract all data property relations which aren't taxonomically classified."""
        relations = []
        data_properties = {prop.reference for prop in self.get_data_properties()}
        
        for prop in data_properties:
            for s, _, o in self.graph.triples((None, prop, None)):
                relation = DataPropertyRelation(s, prop, o, self.graph)
                relations.append(relation)
                
        return relations
    
    def get_non_taxonomic_relations(self):
        """Combine object and data property relations to get all non-taxonomic relations."""
        return self.get_object_property_relations() + self.get_data_property_relations()


    
    #--------------------------------------------------------------------------
    # Axioms Extraction
    #--------------------------------------------------------------------------
    
    # def get_restrictions(self):
    #     """Extract all restrictons from the ontology."""
    #     return self._get_elements_of_type(OWL.Restriction)
    
    def get_some_values_from_restrictions(self):   
        axioms = []
    
        for restriction, _, some_values_from in self.graph.triples((None, OWL.someValuesFrom, None)):
            parent_classes = [s for s, p, o in self.graph.triples((None, RDFS.subClassOf, restriction))]
            
            on_property = self.graph.value(subject=restriction, predicate=OWL.onProperty)
            if on_property:
                for parent_class in parent_classes:
                    axiom = SomeValuesFromRestrictionAxiom(parent_class, restriction, on_property, some_values_from)
                    axioms.append(axiom)
                    
        return axioms


    def get_axioms(self):
        """Extract all axioms from the ontology."""
        return self.get_some_values_from_restrictions()
    

    #--------------------------------------------------------------------------
    # Comparison Method(s)
    #--------------------------------------------------------------------------
    
    @staticmethod
    def compare_entity_names(entity1, entity2):
        return entity1.name() == entity2.name()

    @staticmethod
    def compare_relations(relation1, relation2):
        return relation1.compare_relation(relation2)
    
    @staticmethod
    def compare_axioms(axiom1, axiom2):
        return axiom1 == axiom2
    
    @staticmethod
    def compare_axiom_lists(axioms1, axioms2):
        common = []
        only_in_1 = []
        only_in_2 = []
    
        for axiom in axioms1:
            if axiom in axioms2:
                common.append(axiom)
            else:
                only_in_1.append(axiom)
    
        for axiom in axioms2:
            if axiom not in axioms1:
                only_in_2.append(axiom)
    
        return {
            "common": common,
            "only_in_1": only_in_1,
            "only_in_2": only_in_2
        }

