#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# Ontology Axioms
#------------------------------------------------------------------------------
# Represents various logical axioms within an ontology, including restrictions,
# class equivalences, disjointness, and others.
#==============================================================================

from rdflib import URIRef, BNode


#--------------------------------------------------------------------------
# Base Class for Axioms
#--------------------------------------------------------------------------
class Axiom:
    
    def __str__(self) -> str:
        """String representation of the axiom for display or debugging."""
        raise NotImplementedError("Subclasses must implement this method.")
        
    def __eq__(self, other):
        return isinstance(other, self.__class__)

    def __lt__(self, other):
        return str(self) < str(other)

    def __hash__(self):
        return hash(str(self))


#--------------------------------------------------------------------------
# Restriction Axioms
#--------------------------------------------------------------------------
class RestrictionAxiom(Axiom):
    def __init__(self, parent_class: (URIRef, BNode), restriction_node: (URIRef, BNode)):
        self.parent_class = parent_class
        self.restriction_node = restriction_node
    
    def __eq__(self, other):
        return super().__eq__(other) and self.parent_class == other.parent_class

    def __str__(self):
        return f"Restriction on class: {self.parent_class} with node: {self.restriction_node}"
    
    def __hash__(self):
        return hash((self.parent_class))


class SomeValuesFromRestrictionAxiom(RestrictionAxiom):
    def __init__(self, parent_class: (URIRef, BNode), restriction_node: (URIRef, BNode), on_property: URIRef, some_values_from: (URIRef, BNode)):
        super().__init__(parent_class, restriction_node)
        self.on_property = on_property
        self.some_values_from = some_values_from

    def __eq__(self, other):
        return super().__eq__(other) and self.on_property == other.on_property and self.some_values_from == other.some_values_from

    def __str__(self):
        return f"Restriction on class: {self.parent_class} with node: {self.restriction_node} via property: {self.on_property} with some values from: {self.some_values_from}"

    def __hash__(self):
        return hash((self.parent_class, self.on_property, self.some_values_from))
    
# There are different kinds of restriction axioms, e.g., someValueFrom, allValuesFrom, etc.
# Each type can be implemented as a subclass of RestrictionAxiom if needed.


#--------------------------------------------------------------------------
# Equivalence Axioms
#--------------------------------------------------------------------------
class ClassEquivalenceAxiom(Axiom):
    def __init__(self, classes: [URIRef]):
        self.classes = classes

    def __eq__(self, other):
        return super().__eq__(other) and set(self.classes) == set(other.classes)

    def __hash__(self):
        return hash(tuple(sorted(self.classes)))


#--------------------------------------------------------------------------
# Property Characteristics (e.g., Functional, InverseFunctional, Symmetric)
#--------------------------------------------------------------------------
class PropertyCharacteristicAxiom(Axiom):
    def __init__(self, property_ref: URIRef, characteristic_type: URIRef):
        self.property_ref = property_ref
        self.characteristic_type = characteristic_type

    def __eq__(self, other):
        return super().__eq__(other) and self.property_ref == other.property_ref and self.characteristic_type == other.characteristic_type

    def __hash__(self):
        return hash((self.property_ref, self.characteristic_type))

# Additional axioms, such as subproperty axioms, inverse properties, etc., can be added as needed.
