#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# Ontology Non-Taxonomic Relations
#------------------------------------------------------------------------------
# Represents non-taxonomic relations (object/data properties, domains, ranges, 
# etc.) within an ontology.
#==============================================================================

from rdflib import URIRef

#--------------------------------------------------------------------------
# Base Class for Non-Taxonomic Relations
#--------------------------------------------------------------------------
class NonTaxonomicRelation:
    def __init__(self, source, property, target, graph):
        """
        Initialize a non-taxonomic relation between source and target entities.

        :param source: The starting entity (typically class or property).
        :param property: The relationship property between the source and target.
        :param target: The target entity (typically class or datatype).
        :param graph: The RDF graph from which the relation is extracted.
        """
        self.source = source
        self.property = property
        self.target = target
        self.graph = graph

    def compare_relation(self, other_relation):
        """
        Compare this relation to another relation.

        :param other_relation: The other NonTaxonomicRelation object to compare with.
        :return: True if they represent the same relation, False otherwise.
        """
        return (
            self.source == other_relation.source and 
            self.property == other_relation.property and 
            self.target == other_relation.target
        )

    def __eq__(self, other):
        if isinstance(other, NonTaxonomicRelation):
            return self.compare_relation(other)
        return False

    def __hash__(self):
        return hash((self.source, self.property, self.target))

    def __str__(self):
        return f"({self.source}, {self.property}, {self.target})"


#--------------------------------------------------------------------------
# Object and Data Property Relations
#--------------------------------------------------------------------------
class ObjectPropertyRelation(NonTaxonomicRelation):
    """Represents an 'object property' relation."""
    def __str__(self):
        return super().__str__()

class DataPropertyRelation(NonTaxonomicRelation):
    """Represents a 'data property' relation."""
    def __str__(self):
        return super().__str__()


#--------------------------------------------------------------------------
# Domain and Range Relations
#--------------------------------------------------------------------------
class DomainRelation(NonTaxonomicRelation):
    """Represents a 'domain' relation."""
    def __str__(self):
        return super().__str__()

class RangeRelation(NonTaxonomicRelation):
    """Represents a 'range' relation."""
    def __str__(self):
        return super().__str__()


# Additional relations (like subproperties, inverse properties, etc.) can be added below as needed.
