#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# Ontology Taxonomic Relations
#------------------------------------------------------------------------------
# Represents taxonomic relations (subclass, subproperty, instance of) 
# within an ontology.
#==============================================================================

from rdflib import URIRef

#--------------------------------------------------------------------------
# Base Class for Taxonomic Relations
#--------------------------------------------------------------------------
class TaxonomicRelation:
    def __init__(self, source, target, graph):
        """
        Initialize a taxonomic relation between source and target entities.

        :param source: The starting entity (subclass/individual/property).
        :param target: The target entity (superclass/class/property).
        :param graph: The RDF graph from which the relation is extracted.
        """
        self.source = source
        self.relation_type = ""  # Cet attribut sera défini dans chaque sous-classe
        self.target = target
        self.graph = graph

    def compare_relation(self, other_relation):
        """
        Compare this relation to another relation.

        :param other_relation: The other TaxonomicRelation object to compare with.
        :return: True if they represent the same relation, False otherwise.
        """
        return (
            self.source == other_relation.source and 
            self.target == other_relation.target
        )

    def __eq__(self, other):
        if isinstance(other, TaxonomicRelation):
            return self.compare_relation(other)
        return False

    def __hash__(self):
        return hash((self.source, self.target))

    def __str__(self):
        return f"{self.source} -> {self.target}"
    
    def __lt__(self, other):
        """Compare based on source, relation type, and target entities."""
        if isinstance(other, TaxonomicRelation):
            if self.source != other.source:
                return str(self.source) < str(other.source)
            # If source entities are the same, compare based on relation type
            if self.relation_type != other.relation_type:
                return self.relation_type < other.relation_type
            # If both source entities and relation types are the same, compare based on target entity
            return str(self.target) < str(other.target)
        return NotImplemented



#--------------------------------------------------------------------------
# Subclass Relations
#--------------------------------------------------------------------------
class SubclassOfRelation(TaxonomicRelation):
    """Represents a 'subclass of' relation."""
    def __init__(self, source, target, graph):
        super().__init__(source, target, graph)
        self.relation_type = "subClassOf"

    def __str__(self):
        return f"({self.source}, {self.relation_type}, {self.target})"


#--------------------------------------------------------------------------
# Subproperty Relations
#--------------------------------------------------------------------------
class SubpropertyOfRelation(TaxonomicRelation):
    """Represents a 'subproperty of' relation."""
    def __init__(self, source, target, graph):
        super().__init__(source, target, graph)
        self.relation_type = "subPropertyOf"

    def __str__(self):
        return f"({self.source}, {self.relation_type}, {self.target})"


#--------------------------------------------------------------------------
# Instance Relations
#--------------------------------------------------------------------------
class InstanceOfRelation(TaxonomicRelation):
    """Represents an 'instance of' relation."""
    def __init__(self, source, target, graph):
        super().__init__(source, target, graph)
        self.relation_type = "instanceOf"

    def __str__(self):
        return f"({self.source}, {self.relation_type}, {self.target})"

