#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

"""
Ontology Report Module
------------------------------------------------------------------------------
This module provides a means to generate detailed reports that compare a
reference ontology with a generated ontology. It uses various metrics to
highlight the similarities and differences between the two ontologies,
facilitating a comprehensive evaluation of ontology generation techniques.

The reports can be used for debugging, optimization, and academic purposes,
helping developers and researchers to better understand the quality and
characteristics of generated ontologies in comparison to a reference.

Classes:
    - Report: Main class that represents an ontology comparison report.
"""

class Report:
    """
    A class used to generate detailed reports on the comparison of a 
    reference ontology with a generated ontology.

    Attributes:
    ------------
    reference_ontology : obj
        The reference ontology against which the generated ontology is compared.
    generated_ontology : obj
        The ontology that has been generated and needs to be evaluated.
    metrics : obj
        The metrics used to evaluate the generated ontology against the reference.
    
    Methods:
    ---------
    generate() -> str:
        Produces a string representation of the report, detailing the comparison 
        results and associated metrics.
    """

    def __init__(self, reference_ontology, generated_ontology, metrics):
        """ 
        Initializes the Report with a reference ontology, generated ontology, 
        and comparison metrics.
        """
        self.reference_ontology = reference_ontology
        self.generated_ontology = generated_ontology
        self.metrics = metrics


    def _generate_evaluation_metrics(self, category_name):
        scores_str = f"\n\n== {category_name.replace('_', ' ').capitalize()} ==\n"
    
        category_scores = self.metrics.scores[category_name]
        for element, score in category_scores.items():
            if element != "synthesis":  # We'll print synthesis separately
                scores_str += f"\nMetrics for {element.replace('_', ' ').capitalize()}:\n"
                scores_str += f'Precision: {score._format_metric(score.precision)}\n'
                scores_str += f'Recall: {score._format_metric(score.recall)}\n'
                scores_str += f'F1 Score: {score._format_metric(score.f1)}\n'
                scores_str += f'Total Elements: {score.total_elements}\n'
                scores_str += f'Matched Elements: {score.matched_elements}\n'
                
        # Print synthesis score at the end for the category
        synthesis_score = category_scores["synthesis"]
        scores_str += "\nOverall Metrics (Synthesis):\n"
        scores_str += f'Precision: {synthesis_score._format_metric(synthesis_score.precision)}\n'
        scores_str += f'Recall: {synthesis_score._format_metric(synthesis_score.recall)}\n'
        scores_str += f'F1 Score: {synthesis_score._format_metric(synthesis_score.f1)}\n'
        
        return scores_str


    def generate(self):
        report_str = "=== Ontology Evaluation Report ===\n"

        # Introduction 
        report_str += "\nComparing Reference Ontology with Generated Ontology.\n"

        # Detailed Evaluation Metrics
        for category in self.metrics.scores.keys():
            report_str += self._generate_evaluation_metrics(category)

        return report_str



