#!/usr/bin/python3.10
# -*-coding:Utf-8 -*-

"""
ontoScorer: Batch Module for comparing multiple pairs of reference and generated ontologies.
------------------------------------------------------------------------------------------
This module facilitates the batch processing of ontology pairs, computing metrics and
generating comparison reports for each pair and a summary report for all.
"""

import os
from report import Report
from metrics import Metrics
from metric_score import Score
from scorer import OntoScorer

class ScorerBatch:
    """
    The ScorerBatch class is used to compare multiple pairs of reference and generated ontologies.
    
    Attributes:
    - ontology_pairs (list): List of tuples, where each tuple contains the paths to the reference and generated ontology.
    - output_dir (str): Path to the directory where the reports will be written.
    - results (list): List of metrics for each pair of ontologies.
    """

    def __init__(self, ontology_pairs, equivalent_prefix, output_dir):
        """
        Initializes the ScorerBatch with a list of ontology pairs and output directory.
        
        Args:
        - ontology_pairs (list): List of tuples. Each tuple contains paths to the reference and generated ontology.
        - equivalent_prefix (str): Prefix to handle equivalent terms or concepts.
        - output_dir (str): Path to the directory where the reports will be written.
        """
        self.ontology_pairs = ontology_pairs
        self.equivalent_prefix = equivalent_prefix
        self.output_dir = output_dir
        self.results = {}

        # Create the output directory if it doesn't exist
        os.makedirs(self.output_dir, exist_ok=True)


    def compute_all_metrics(self):
        """
        Computes metrics for each pair of ontologies and stores them in the results dictionary.
        """
        for ref_onto, gen_onto in self.ontology_pairs:
            scorer = OntoScorer(ref_onto, gen_onto, self.equivalent_prefix)
            scorer.compute_metrics()
            self.results[(ref_onto, gen_onto)] = scorer.metrics 


    def generate_all_reports(self):
        """
        Generates and writes a report for each pair of ontologies to the output directory.
        """
        for idx, ((ref_onto, gen_onto), metrics) in enumerate(self.results.items()):
            report_content = f"Metrics for Ontology Pair {idx + 1}:\n"
            report = Report(ref_onto, gen_onto, metrics)
            report_content += report.generate() + "\n"
            report_content += "-"*80 + "\n"
        
            report_filename = os.path.join(self.output_dir, f'report_{idx + 1}.txt')
            with open(report_filename, 'w') as f:
                f.write(report_content)


    def collect_scores_for_metric(self, metric_category, metric_name):
        scores = []
        for _, metrics in self.results.items():
            score = metrics.scores[metric_category][metric_name]
            scores.append(score)
        return scores


    def generate_summary_report(self):
        summary_content = "Summary Report for All Ontologies\n"
        summary_content += "="*80 + "\n"
        
        # Section pour les statistiques globales
        summary_content += "Overall Statistics:\n"
        for category, metrics_dict in Metrics().scores.items():
            for metric_name, _ in metrics_dict.items():
                score_objects = self.collect_scores_for_metric(category, metric_name)
                
                aggregated_score = Score.aggregate_f1(score_objects)
                precision_str = f"{aggregated_score.precision:.4f}" if aggregated_score.precision is not None else "N/A"
                recall_str = f"{aggregated_score.recall:.4f}" if aggregated_score.recall is not None else "N/A"
                f1_str = f"{aggregated_score.f1:.4f}" if aggregated_score.f1 is not None else "N/A"
    
                summary_content += f"{category} - {metric_name}:\n"
                summary_content += f"\tPrecision: {precision_str}\n"
                summary_content += f"\tRecall: {recall_str}\n"
                summary_content += f"\tF1 Score: {f1_str}\n"
                summary_content += "-"*40 + "\n"
    
        # Section pour les résultats par paire d'ontologies
        summary_content += "Detailed Results:\n"
        for idx, ((ref_onto, gen_onto), metrics) in enumerate(self.results.items()):
            summary_content += f"Metrics for Ontology Pair {idx + 1}:\n"
            report = Report(ref_onto, gen_onto, metrics)
            summary_content += report.generate() + "\n"
            summary_content += "-"*80 + "\n"
    
        summary_filename = os.path.join(self.output_dir, 'summary_report.txt')
        with open(summary_filename, 'w') as f:
            f.write(summary_content)
