#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

"""
test_metrics: Metrics Testing Module
------------------------------------------------------------------------------
Contains tests for verifying functionality of the Metrics class.
"""

import unittest
import os
from context import ontoScorer
from ontoScorer.ontology import Ontology
from ontoScorer.metrics import Metrics

#--------------------------------------------------------------------------
# Metrics Test
#--------------------------------------------------------------------------
class TestMetrics(unittest.TestCase):

    def setUp(self):
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.ontology2_path = f"{DATA_FOLDER_PATH}/ontology_b.ttl"
        self.ontology3_path = f"{DATA_FOLDER_PATH}/ontology_c.ttl"
        
        self.equivalent_prefix = [
            ("base", "https://reference.tetras-libre.fr/base-ontology#", "https://tenet.tetras-libre.fr/base-ontology#"),
            ("result", "https://reference.tetras-libre.fr/expected-result#", "https://tenet.tetras-libre.fr/extract-result#")
        ]
        
        self.onto1 = Ontology(self.ontology1_path, self.equivalent_prefix)
        self.onto2 = Ontology(self.ontology2_path, self.equivalent_prefix)
        self.onto3 = Ontology(self.ontology3_path, self.equivalent_prefix)
        
        self.metrics = Metrics()


    def test_computes_entity_scores(self):
        self.metrics.compute_entity_scores(self.onto1, self.onto2)
        self.verify_scores(self.metrics.scores["entities"])

    def test_computes_taxonomic_relation_scores(self):
        self.metrics.compute_taxonomic_relation_scores(self.onto1, self.onto2)
        self.verify_scores(self.metrics.scores["taxonomic_relations"])

    def test_computes_non_taxonomic_relation_scores_1(self):
        self.metrics.compute_non_taxonomic_relation_scores(self.onto1, self.onto2)
        self.verify_scores(self.metrics.scores["non_taxonomic_relations"])

    def test_computes_non_taxonomic_relation_scores_2(self):
        self.metrics.compute_non_taxonomic_relation_scores(self.onto3, self.onto3)
        self.verify_scores(self.metrics.scores["non_taxonomic_relations"])

    def test_computes_axiom_scores(self):
        self.metrics.compute_axiom_scores(self.onto1, self.onto2)
        self.verify_scores(self.metrics.scores["axioms"])

    def verify_scores(self, score_category):
        for element, score in score_category.items():
            if score.total_elements == 0:
                self.assertIsNone(score.precision, f"Precision for {element} should be None when no elements are present")
                self.assertIsNone(score.recall, f"Recall for {element} should be None when no elements are present")
                self.assertIsNone(score.f1, f"F1 score for {element} should be None when no elements are present")
            else:
                self.assertTrue(0 <= score.precision <= 1, f"Invalid precision for {element}")
                self.assertTrue(0 <= score.recall <= 1, f"Invalid recall for {element}")
                self.assertTrue(0 <= score.f1 <= 1, f"Invalid F1 score for {element}")

    def test_print_scores(self):
        self.metrics.compute_entity_scores(self.onto1, self.onto2)
        self.metrics.compute_taxonomic_relation_scores(self.onto1, self.onto2)
        self.metrics.compute_non_taxonomic_relation_scores(self.onto3, self.onto3)
        self.metrics.compute_axiom_scores(self.onto1, self.onto2)
        print()
        self.metrics.print_scores()


#--------------------------------------------------------------------------
# Main Unit Test Run
#--------------------------------------------------------------------------
if __name__ == "__main__":
    unittest.main()
