#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# test_ontology: Ontology Testing Module
#------------------------------------------------------------------------------
# Contains tests for verifying functionality of the Ontology class.
#==============================================================================

import unittest
import os
from context import ontoScorer

from ontoScorer.ontology import Ontology
from ontoScorer.ontology_axioms import *
from ontoScorer.ontology_nontaxonomic_relations import *
from ontoScorer.ontology_taxonomic_relations import *


#--------------------------------------------------------------------------
# Ontology Test (Base)
#--------------------------------------------------------------------------
class TestOntology(unittest.TestCase):

    def setUp(self):
        # Test ontology paths
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.ontology2_path = f"{DATA_FOLDER_PATH}/ontology_b.ttl"
        self.onto1 = Ontology(self.ontology1_path)
        self.onto2 = Ontology(self.ontology2_path)

    def test_load_ontology(self):
        self.assertIsNotNone(self.onto1.graph)
        self.assertIsNotNone(self.onto2.graph)


#--------------------------------------------------------------------------
# Ontology Test (Entity Extraction)
#--------------------------------------------------------------------------
class TestOntology(unittest.TestCase):

    def setUp(self):
        # Test ontology paths
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.ontology2_path = f"{DATA_FOLDER_PATH}/ontology_b.ttl"
        self.onto1 = Ontology(self.ontology1_path)
        self.onto2 = Ontology(self.ontology2_path)

    def test_get_classes(self):
        classes1_names = {c.name() for c in self.onto1.get_classes()}
        classes2_names = {c.name() for c in self.onto2.get_classes()}
        
        self.assertIn("gravitation-bind-system", classes1_names)
        self.assertIn("gravitation-bind-system", classes1_names)
        self.assertIn("object-orbit-hasManner-direct-sun", classes1_names)
        self.assertIn("object-orbit-hasManner-not-direct-sun", classes1_names)
        self.assertIn("gravitation", classes1_names)
        self.assertIn("system-hasPart-object", classes1_names)
        self.assertIn("system-hasPart-sun", classes1_names)
        self.assertIn("object", classes1_names)
        self.assertIn("sun", classes1_names)
        self.assertIn("system", classes1_names)
        self.assertNotIn("system-isBindBy-gravitation", classes1_names)
        self.assertIn("system-isBindBy-gravitation", classes2_names)

    def test_get_object_properties(self):
        object_properties_names = {op.name() for op in self.onto1.get_object_properties()}
        
        self.assertIn("direct", object_properties_names)
        self.assertIn("hasManner", object_properties_names)
        self.assertIn("not-direct", object_properties_names)
        self.assertIn("bind", object_properties_names)
        self.assertIn("orbit-hasManner-direct", object_properties_names)
        self.assertIn("orbit-hasManner-not-direct", object_properties_names)
        self.assertIn("hasPart", object_properties_names)
        self.assertIn("orbit", object_properties_names)

    def test_get_data_properties(self):
        data_properties_names_1 = {dp.name() for dp in self.onto1.get_data_properties()}
        data_properties_names_2 = {dp.name() for dp in self.onto2.get_data_properties()}
        self.assertEqual(len(data_properties_names_1), 0)
        self.assertEqual(len(data_properties_names_2), 0)

    def test_get_individuals(self):
        individuals_names = {ind.name() for ind in self.onto1.get_individuals()}
        self.assertEqual(len(individuals_names), 1)
        self.assertIn("SolarSystem", individuals_names)
        self.assertNotIn("gravitation", individuals_names)


#--------------------------------------------------------------------------
# Ontology Test (Taxonomic Relations)
#--------------------------------------------------------------------------
class TestOntologyTaxonomicRelations(unittest.TestCase):

    def setUp(self):
        # Test ontology paths
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.ontology2_path = f"{DATA_FOLDER_PATH}/ontology_b.ttl"
        self.onto1 = Ontology(self.ontology1_path)
        self.onto2 = Ontology(self.ontology2_path)

    def test_subclass_relation_extraction_1(self):
        relations = self.onto1.get_subclass_relations()
        for r in relations:
            self.assertIsInstance(r, SubclassOfRelation, f"Unexpected relation type: {type(r)} for relation: {r}")

    def test_subclass_relation_extraction_2(self):
        relations = self.onto1.get_taxonomic_relations()
        subclasses = [rel for rel in relations if isinstance(rel, SubclassOfRelation)]
        self.assertTrue(any(rel.source.endswith("gravitation-bind-system") and rel.target.endswith("gravitation") for rel in subclasses))
        self.assertTrue(any(rel.source.endswith("object-orbit-hasManner-direct-sun") and rel.target.endswith("object") for rel in subclasses))
        

    def test_subproperty_relation_extraction_1(self):
        relations = self.onto1.get_subproperty_relations()
        for r in relations:
            self.assertIsInstance(r, SubpropertyOfRelation, f"Unexpected relation type: {type(r)} for relation: {r}")

    def test_subproperty_relation_extraction_2(self):
        relations = self.onto1.get_taxonomic_relations()
        subproperties = [rel for rel in relations if isinstance(rel, SubpropertyOfRelation)]
        self.assertTrue(any(rel.source.endswith("#direct") and rel.target.endswith("Out_ObjectProperty") for rel in subproperties))
        self.assertTrue(any(rel.source.endswith("#orbit-hasManner-direct") and rel.target.endswith("#orbit") for rel in subproperties))
    

    def test_instance_relation_extraction_1(self):
        relations = self.onto1.get_instance_relations()
        for r in relations:
            self.assertIsInstance(r, InstanceOfRelation, f"Unexpected relation type: {type(r)} for relation: {r}")

    def test_instance_relation_extraction_2(self):
        relations = self.onto1.get_taxonomic_relations()
        instances = [rel for rel in relations if isinstance(rel, InstanceOfRelation)]
        self.assertTrue(any(rel.source.endswith("#SolarSystem") and rel.target.endswith("system") for rel in instances))


    def test_get_taxonomic_relations(self):
        relations = self.onto1.get_taxonomic_relations()
        for r in relations:
            self.assertIsInstance(r, (SubclassOfRelation, SubpropertyOfRelation, InstanceOfRelation),
                                  f"Unexpected relation type: {type(r)} for relation: {r}")


#--------------------------------------------------------------------------
# Ontology Test (Non-Taxonomic Relations)
#--------------------------------------------------------------------------
class TestOntologyNonTaxonomicRelations(unittest.TestCase):

    def setUp(self):
        # Test ontology paths
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_c.ttl"
        self.onto1 = Ontology(self.ontology1_path)

    def test_object_property_relations_1(self):
        relations = self.onto1.get_object_property_relations()
        for r in relations:
            self.assertIsInstance(r, ObjectPropertyRelation, 
                                  f"Unexpected relation type: {type(r)} for relation: {r}")

    def test_object_property_relations_2(self):
        relations = self.onto1.get_non_taxonomic_relations()
        instances = [rel for rel in relations if isinstance(rel, ObjectPropertyRelation)]
        self.assertTrue(any(rel.source.endswith("SolarSystem") 
                            and rel.property.endswith("contains") 
                            and rel.target.endswith("SomeSmallerObjectInstance") 
                            for rel in instances))


    def test_data_property_relations_1(self):
        relations = self.onto1.get_data_property_relations()
        for r in relations:
            self.assertIsInstance(r, DataPropertyRelation, 
                                  f"Unexpected relation type: {type(r)} for relation: {r}")

    def test_get_non_taxonomic_relations(self):
        relations = self.onto1.get_non_taxonomic_relations()
        for r in relations:
            self.assertIsInstance(r, (ObjectPropertyRelation, DataPropertyRelation, DomainRelation, RangeRelation),
                                  f"Unexpected relation type: {type(r)} for relation: {r}")


#--------------------------------------------------------------------------
# Ontology Test (Axioms)
#--------------------------------------------------------------------------
class TestOntologyAxioms(unittest.TestCase):

    def setUp(self):
        # Test ontology paths
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.onto1 = Ontology(self.ontology1_path)

    def test_get_some_values_from_restrictions(self):
        axioms = self.onto1.get_some_values_from_restrictions()
        self.assertEqual(len(axioms), 5, "Expected 5 SomeValuesFromRestrictionAxioms.")
        axiom = axioms[0]
        self.assertIsInstance(axiom, SomeValuesFromRestrictionAxiom)
        # self.assertTrue(any(rel.on_property.endswith("#orbit-hasManner-not-direct") 
        #                     and rel.some_values_from.endswith("#sun") 
        #                     for rel in axioms))
        self.assertTrue(any(rel.on_property.endswith("orbit-hasManner-direct") 
                            and rel.some_values_from.endswith("sun") 
                            for rel in axioms))
        self.assertTrue(any(rel.on_property.endswith("bind") 
                            and rel.some_values_from.endswith("system") 
                            and rel.parent_class.endswith("gravitation-bind-system")
                            for rel in axioms))


#--------------------------------------------------------------------------
# Tests for Comparison Methods
#--------------------------------------------------------------------------

class TestOntologyComparisonMethods(unittest.TestCase):

    def setUp(self):
        
        DATA_FOLDER_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/test_data'
        self.ontology1_path = f"{DATA_FOLDER_PATH}/ontology_a.ttl"
        self.ontology2_path = f"{DATA_FOLDER_PATH}/ontology_b.ttl"
        self.ontology3_path = f"{DATA_FOLDER_PATH}/ontology_c.ttl"
        
        self.equivalent_prefix = [
            ("base", "https://reference.tetras-libre.fr/base-ontology#", "https://tenet.tetras-libre.fr/base-ontology#"),
            ("result", "https://reference.tetras-libre.fr/expected-result#", "https://tenet.tetras-libre.fr/extract-result#")
        ]
        
        self.onto1 = Ontology(self.ontology1_path, self.equivalent_prefix)
        self.onto2 = Ontology(self.ontology2_path, self.equivalent_prefix)
        self.onto3 = Ontology(self.ontology3_path, self.equivalent_prefix)

    def test_compare_entity_names(self):
        entity1 = sorted(self.onto1.get_classes())[0]
        entity2 = sorted(self.onto2.get_classes())[0]
        entity3 = sorted(self.onto2.get_classes())[1]
        self.assertTrue(Ontology.compare_entity_names(entity1, entity2),
                        f'{entity1.name()} | {entity2.name()}')
        self.assertFalse(Ontology.compare_entity_names(entity1, entity3))

    def test_compare_relations_1(self):
        relation1 = sorted(self.onto1.get_taxonomic_relations())[0]
        relation2 = sorted(self.onto2.get_taxonomic_relations())[0]
        relation3 = sorted(self.onto2.get_taxonomic_relations())[1]
        self.assertTrue(Ontology.compare_relations(relation1, relation2),
                        f'{relation1} | {relation2}')
        self.assertFalse(Ontology.compare_relations(relation1, relation3))

    def test_compare_relations_2(self):
        relation1 = self.onto3.get_non_taxonomic_relations()[0]
        relation2 = self.onto3.get_non_taxonomic_relations()[0]
        # relation3 = self.onto3.get_non_taxonomic_relations()[1]
        self.assertTrue(Ontology.compare_relations(relation1, relation2),
                        f'{relation1} | {relation2}')
        # self.assertFalse(Ontology.compare_relations(relation1, relation3))

    def test_compare_axioms(self):
        axiom1 = sorted(self.onto1.get_axioms())[0]
        axiom2 = sorted(self.onto2.get_axioms())[0]
        axiom3 = sorted(self.onto2.get_axioms())[1]
        self.assertTrue(Ontology.compare_axioms(axiom1, axiom2),
                        f'{axiom1} | {axiom2}')
        self.assertFalse(Ontology.compare_axioms(axiom1, axiom3))



#--------------------------------------------------------------------------
# Main Unit Test Run
#--------------------------------------------------------------------------
if __name__ == "__main__":
    unittest.main()
